/*
	File:		librTemplate.r

	Contains:	Template for a 'libr' resource

	Copyright:	 1991-1993 by Apple Computer, Inc., all rights reserved.

*/

#ifndef __LIBRTEMPLATE__
#define __LIBRTEMPLATE__

/*
	Template for a 'libr' resource which is located in each library. There is one
	per library so there could be more than one in a library file. It gives information
	about the library including what classes it implements.
*/

type 'libr' {
	array LibrLine {					/* information for a library */
		cstring;						/* Library id */
		align word;
		string[4];						/* code resource type */
		hex byte;						/* libr template major version */
		hex byte;						/* libr template minor version */
		hex byte;						/* Major version */
		hex byte;						/* Minor revision */
		hex byte;						/* Development stage */
		hex byte;						/* Release within stage */
		hex byte;						/* Reserved in v1 (0)	*/
		hex byte;						/* Compiler type		*/
		integer;						/* perClientDataSize in v1 (0 = default) */
		longint;						/* HeapSize in v1 (0 = default) */
		longint 	preload = 0x01,
					clientPool = 0x02,
					nosegunload = 0x04,
					loaddeps = 0x08,
					forcedeps = 0x10,
					loadself  = 0x20,
					notSystem7 = 0x10000,
					notSystem6 = 0x20000,
					notVMOn = 0x40000,
					notVMOff= 0x80000,
					hasFPU=0x100000,
					hasNoFPU=0x200000,
					not68000=0x400000,
					not68020=0x800000,
					not68030=0x1000000,
					not68040=0x2000000;	/* flags for the code resource */
		integer = $$Countof(ClassIDs);	/* # class IDs */
		array ClassIDs{					/* array of classIds the library implements */
			longint	preload = 0x01,
					newobject = 0x02,
					isFunctionSet = 0x04;	/* flags for the class */
			integer;						/* current version		*/
			integer;						/* minimum version		*/
			cstring;						/* the class id string	*/
			align WORD;
			integer = $$CountOf(ParentIDs);
			array ParentIDs {
				cstring;
			};							/* the parent class id string */
			align WORD;
		};
	};
};		

#endif