/*
	File:		TSimpleListExample.cp

	Contains:	This module shows an example use of the TSimpleList class.

	Copyright:	 1993 by Apple Computer, Inc., all rights reserved.

*/

#include "TInitSLM.h"			// the TInitSLM class and SLM include files
#include "TListExample.h"

///
///	CONSTANTS
///

const unsigned	kMaxPoints = 20;		// maximum number of points
const unsigned	kNoPoints = 5;			// number of points for now

///
///	PROTOTYPES
///

static void build_list_of_points( TSimpleList* simpleList );
static void remove_list_of_points( TSimpleList*	simpleList );
static void disp_list_of_points( TSimpleList* simpleList );

/*
	main 
	
	This examples creates a list of TPoints using the TSimpleList. Since it not creating
	a TLink seperately, it creates a link pool for the list, and uses AddFirst
	calls which requires only a pointer to an object. Otherwise we could've
	created our own TLink and used AddLinkFirst. Once the list has
	been created it uses an iterator to index thru the list and display the contents
	of the list. Once displayed it deletes the list and its items.
*/

main() 
{	
	TInitSLM initLibraryManager;			// initialize the shared library manager
	
	if( initLibraryManager.Failed() )	// If we mfailed, let's go home
		return 1;

	// Allocate the pool for TSimpleList's TLinks. Since the size of the blocks
	// is constant, it's better to use a TChunkyPool.
	size_t poolsize = kMaxPoints * (sizeof(TLink) + kChunkyPoolChunkOverhead);
	TChunkyPool	*listpool = new (poolsize,kSystemZone) TChunkyPool(sizeof(TLink));
		
	TSimpleList *listofpts = new TSimpleList(listpool);
	
	if( listofpts ) {					// make sure we allocated the list OK!

		build_list_of_points( listofpts ); // obvious what I am doing!
		
		if( ! listofpts->IsEmpty() ) {	// make sure list is not empty
			cout << "Total no. of points = " << listofpts->Count() << "\n" << endl;

			// let's go through the list of points and print the values
			disp_list_of_points(listofpts);
			
			// now let's remove and delete the objects
			remove_list_of_points(listofpts);
		}
		else
			cout << "Empty lists\n" << endl;
		delete listofpts;				// we are done with listofpts
		delete listpool;				// free the pool
	}
	return 0;
}

/*
	build_list_of_points
	
	Create a list of TPoint objects
*/

static void build_list_of_points( TSimpleList*	simpleList )
{
	for( int i=0; i< kNoPoints; i++ ) {
		TPoint *point = new TPoint(i,i*10);	// create a point
		simpleList->AddLast( point );
	}
}

/*
	remove_list_of_points
	
	Remove the list of points. Here you can use RemoveAll or remove each item one by
	one. We remove then one by one here since we also have to delete them.
*/

static void remove_list_of_points( TSimpleList*	simpleList )
{
		TPoint* point;
		while( point = (TPoint*)simpleList->RemoveFirst())
			delete point;
}

/*
	disp_list_of_points
	
	Use a TListIterator to iterate thru the list and display the values for each point
*/

static void disp_list_of_points( TSimpleList* simpleList )
{
	TListIterator	iter(simpleList);
	TPoint			*point;
	short			count = 0;
	while( point = (TPoint*)iter.Next() ) {
		count++;
		cout << "point[" << count << "] = " << "(" << (unsigned)point->GetPoint().h << "," <<
				(unsigned)point->GetPoint().v << ")" << "\n" << endl;
	}
}
