/*
	File:		InspectorMain.r

	Contains:	Resources for Inspector

	Copyright:	 1991-1993 by Apple Computer, Inc., all rights reserved.

*/


#include "SysTypes.r"
#include "Types.r"

INCLUDE "Inspector.obj";

#include "VersionResource.r"

#include "SimpleCommon.h"


#define kPrefSize				100
#define kMinSize				75

/* here is the quintessential MultiFinder friendliness device, the SIZE resource */

resource 'SIZE' (-1) {
	dontSaveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	multiFinderAware,	/* this says we do our own activate/deactivate; don't fake us out */
	backgroundAndForeground, /* this is definitely note a background-only application! */
	dontGetFrontClicks,	/* change this is if you want "do first click" behavior like the Finder */
	ignoreChildDiedEvents,
	is32BitCompatible,
	reserved, reserved, reserved, reserved,
	reserved, reserved, reserved,
	kPrefSize * 1024,
	kMinSize * 1024
};


type 'MOOT' as 'STR ';


resource 'MOOT' (0) {
	"ASLM Test Application"
};


resource 'BNDL' (128) {
	'MOOT',
	0,
	{
		'ICN#',
		{
			0, 128
		},
		'FREF',
		{
			0, 128
		}
	}
};


resource 'FREF' (128) {
	'APPL',
	0,
	""
};


resource 'ICN#' (128) {
	{ /* array: 2 elements */
		/* [1] */
		$"04 30 40 00 0A 50 A0 00 0B 91 10 02 08 22 08 03"
		$"12 24 04 05 20 28 02 09 40 10 01 11 80 0C 00 A1"
		$"80 03 FF C2 7E 00 FF 04 01 00 7F 04 03 00 1E 08"
		$"04 E0 00 0C 08 E0 00 0A 10 E0 00 09 08 C0 00 06"
		$"04 87 FE 04 02 88 01 04 01 88 00 84 00 88 00 44"
		$"00 88 00 44 00 88 00 C4 01 10 01 88 02 28 03 10"
		$"01 C4 04 E0 00 02 08 00 73 BF FB EE 4C A2 8A 2A"
		$"40 AA AA EA 52 AA AA 24 5E A2 8A EA 73 BE FB 8E",
		/* [2] */
		$"04 30 40 00 0E 70 E0 00 0F F1 F0 02 0F E3 F8 03"
		$"1F E7 FC 07 3F EF FE 0F 7F FF FF 1F FF FF FF BF"
		$"FF FF FF FE 7F FF FF FC 01 FF FF FC 03 FF FF F8"
		$"07 FF FF FC 0F FF FF FE 1F FF FF FF 0F FF FF FE"
		$"07 FF FF FC 03 FF FF FC 01 FF FF FC 00 FF FF FC"
		$"00 FF FF FC 00 FF FF FC 01 FF FF F8 03 EF FF F0"
		$"01 C7 FC E0 00 03 F8 00 73 BF FB EE 7F BE FB EE"
		$"7F BE FB EE 7F BE FB E4 7F BE FB EE 73 BE FB 8E"
	}
};


resource 'STR#' (kInspectorMainErrStrings, purgeable) {
	{
	"Could not load the InspectorLibrary. Inspector functionality will be minimal";
	"InitLibraryManager failed. Inspector functionality will be minimal"
	}
};

/* Resources for the SimpleProgram NV routine */

/* we use an MBAR resource to conveniently load all the menus */

resource 'MBAR' (rMenuBar) {
	{ mApple, mFile, mEdit};
};

resource 'MENU' (mApple) {
	mApple, textMenuProc,
	0b1111111111111111111111111111101,	/* disable dashed line, enable About and DAs */
	enabled, apple,
	{
		"About Inspector",
			noicon, nokey, nomark, plain;
		"-",
			noicon, nokey, nomark, plain
	}
};

resource 'MENU' (mFile) {
	mFile, textMenuProc,
	0b0000000000000000000000000001000,	/* enable Quit only, program enables others */
	enabled, "File",
	{
		"Load ASLM",
			noicon, "L", nomark, plain;
		"Unload ASLM",
			noicon, "U", nomark, plain;
		"Goto Real Program",
			noicon, "G", nomark, plain;
		"Quit",
			noicon, "Q", nomark, plain
	}
};


resource 'MENU' (mEdit) {
	mEdit, textMenuProc,
	0b0000000000000000000000000000000,	/* disable everything, program does the enabling */
	enabled, "Edit",
	 {
		"Undo",
			noicon, "Z", nomark, plain;
		"-",
			noicon, nokey, nomark, plain;
		"Cut",
			noicon, "X", nomark, plain;
		"Copy",
			noicon, "C", nomark, plain;
		"Paste",
			noicon, "V", nomark, plain;
		"Clear",
			noicon, nokey, nomark, plain
	}
};


/* this ALRT and DITL are used as an error screen */
resource 'ALRT' (rUserAlert, purgeable) {
	{40, 20, 180, 330},
	rUserAlert,
	{ /* array: 4 elements */
		/* [1] */
		OK, visible, silent,
		/* [2] */
		OK, visible, silent,
		/* [3] */
		OK, visible, silent,
		/* [4] */
		OK, visible, silent
	}
};

resource 'DITL' (rUserAlert, purgeable) {
	{ /* array DITLarray: 3 elements */
		/* [1] */
		{110, 220, 130, 300},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 60, 90, 300},
		StaticText {
			disabled,
			"Error. ^0."
		},
		/* [3] */
		{8, 8, 40, 40},
		Icon {
			disabled,
			2
		}
	}
};


/* this ALRT and DITL are used as an About screen */

resource 'ALRT' (rAboutAlert, purgeable) {
	{40, 20, 160, 330 }, rAboutAlert, {
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent;
		OK, visible, silent
	};
};

resource 'DITL' (rAboutAlert, purgeable) {
	{ /* array DITLarray: 5 elements */
		/* [1] */
		{88, 224, 108, 304},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{8, 8, 24, 304 },
		StaticText {
			disabled,
			"ASLM Inspector Application"
		},
		/* [3] */
		{32, 8, 48, 237},
		StaticText {
			disabled,
			"Copyright  1991-1993 Apple Computer"
		},
		/* [4] */
		{56, 8, 72, 136},
		StaticText {
			disabled,
			"Brought to you by:"
		},
		/* [5] */
		{80, 24, 112, 220},
		StaticText {
			disabled,
			"The ASLM Team"
		}
	}
};
