/*
	File:		List.h

	Contains:	Interface for TList class, a front end to the list manager.

	Copyright:	 1991-1993 by Apple Computer, Inc., all rights reserved.

*/


#ifndef __LIST__
#define __LIST__

#ifndef __LISTS__
#include <Lists.h>
#endif

#ifndef __LIBRARYMANAGER__
#include <LibraryManager.h>
#endif

#define kTListID "appl:insp$TList,1.1"

class TList : public TDynamic
{
public:
					TList();
					TList(const Rect *rView, const Rect *dataBounds, Point cSize,
						short theProc, WindowPtr theWindow, Boolean drawIt, Boolean hasGrow,
						Boolean scrollHoriz, Boolean scrollVert);
					~TList();
					
	virtual	short	AddColumn(short count,short colNum);
	virtual	short	AddRow(short count, short rowNum);
	virtual	short	AddRow(short count);
	virtual	void	DelColumn(short count,short colNum);
	virtual	void	DelRow(short count,short rowNum);

	virtual	void	AddToCell(const void *dataPtr,short dataLen,Cell theCell);
	virtual	void	ClrCell(Cell theCell);
	virtual	void	GetCell(void *dataPtr,short *dataLen,Cell theCell);
	virtual	void	SetCell(const void *dataPtr,short dataLen,Cell theCel);
	virtual	void	CellSize(Point cSize);
	virtual	Boolean	GetSelect(Boolean next,Cell *theCell);
	virtual	void	SetSelect(Boolean setIt,Cell theCell);
	
	virtual	Boolean	Click(Point pt,short modifiers);
	virtual	Cell	LastClick();

	virtual	void	Find(short *offset,short *len,Cell theCell);
	virtual	Boolean	NextCell(Boolean hNext,Boolean vNext,Cell *theCell);
	virtual	void	CellRect(Rect *cellRect,Cell theCell);
	virtual	Boolean	Search(const void *dataPtr,short dataLen,SearchProcPtr searchProc,
							Cell *theCell);
	virtual	void	Size(short listWidth,short listHeight);

	virtual	void	Draw(Cell theCell);
	virtual	void	DoDraw(Boolean drawIt);
	virtual	void	Scroll(short dCols,short dRows);
	virtual	void	AutoScroll();
	virtual	void	Update(RgnHandle theRgn);
	virtual	void	Activate(Boolean act);
			
	virtual	void	SetSelFlags(char theFlags);
	virtual	short	GetLastRow();
	virtual	short	GetLastColumn();
	virtual	ListHandle	GetList();
	virtual	Rect	GetListRect();
	
private:
	virtual	void		InitList(const Rect *rView, const Rect *dataBounds, Point cSize,
							short theProc, WindowPtr theWindow, Boolean drawIt, Boolean hasGrow,
							Boolean scrollHoriz, Boolean scrollVert);

protected:
	ListHandle			fList;
};

#endif
